#!/usr/bin/perl

# Convert long filenames in HTML documents to short ones.
# Copyright  1999 Marko Mkel <Marko.Makela@HUT.FI>.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.

($#ARGV >= 2 && !($#ARGV % 2)) || die
    "Converts long filenames in HTML documents to short ones,\n".
    "using TRANS.TBL information.\n".
    "Usage: $0 /cdrom long1.html SHORT1.HTM ... long2.html SHORT2.HTM\n";

$cdrom = shift @ARGV;
$cdrom .= '/' unless ($cdrom =~ /\/$/);
%fixedmap = ();
while ($#ARGV > 0)
{
    $longname = shift @ARGV;
    $shortname = shift @ARGV;
    $fixedmap{$longname} = $shortname;
}

%name = ("" => "");
@DIRS = ();
$currentfile = ""; # current file name, for diagnostics
&readtable ("");
&convert;

# Read the TRANS.TBL files, generating a map from long names to short names
sub readtable
{
    local ($path) = @_;
    open (TABLE, $cdrom.$path."TRANS.TBL") ||
	die $cdrom.$path."TRANS.TBL not found\n";
    while (<TABLE>)
    {
	if (/^([FD]) ([^ ]+) *\t(.*)$/)
	{
	    $type = $1;
	    $longname = $3;
	    ($shortname = $fixedmap{$longname}) ||
		($shortname = $2) =~ s/;[1-9][0-9]*$//;
	    exists $name{$path} || die "$path not found\n";
	    $name{$path.$longname} = $name{$path}.$shortname;
	    if ($type eq 'D') {
		$name{$path.$longname."/"} = $name{$path}.$shortname."/";
		push @DIRS, $path.$longname."/";
	    }
	}
    }
    close (TABLE);

    while ($path = shift @DIRS)
    {
	&readtable ($path);
    }
}

# Convert the HTML documents
sub convert
{
    for $path (grep (/^$/ || /\/$/, keys %name))
    {
	for (keys %fixedmap)
	{
	    $currentfile = $path.$_; # $cdrom.$path.$_
	    if (open (HTML, $currentfile)) {
		open (HTM, ">$path".$fixedmap{$_}) ||
		    die "Unable to create $path$fixedmap{$_}\n";
		chmod 0660, $path.$fixedmap{$_} ||
		    die "chmod 660 $path.$fixedmap{$_} failed\n";

		while (<HTML>)
		{
		    s/(HREF|SRC)=\"([^\"\#]+)(?=[\"\#])/$1.'="'.shorten($2,$path)/gei;
		    print HTM;
		}

		close (HTM);
		close (HTML);
	    }
	}
    }
}

# Shorten the components of a name
sub shorten
{
    local ($name, $path) = @_;

    $_ = $name;
    tr/+/ /;
    s/%([0-9a-f][0-9a-f])/pack ("C",hex $1)/gei;

    if (/^(http|ftp|nntp|news|mailto):/i)
    {
	return $name;
    }

    # Eliminate (../)* from the beginning of the name
    local ($dots) = "";
    while (s/^\.\.\///) {
	if ($path !~ s/\/[^\/]+\/$/\// &&
	    $path !~ s/^[^\/]+\/$//)
	{
	    warn "$currentfile: unable to translate \"$name\"\n";
	    return $name;
	}
	$dots .= "../";
    }

    unless (exists $name{$path.$_})
    {
	warn "$currentfile: no translation for \"$path$_\" found\n" ;
	return $name;
    }
    $_ = $name{$path.$_};
    die unless s/^\Q$name{$path}\E//;
    return $dots.$_;
}
