#!/bin/sh

# This script creates an ISO 9660 CD-ROM image with Rock Ridge and Joliet
# extensions of the http://www.funet.fi/pub/cbm/ file archive.
# Written in 2002 by Marko Mkel.  This script is in the public domain.

# In order to use this script, you will need:
#  a Unix-like system whose mount(8) supports ISO 9660 and the loopback device
#  rsync (tested with 2.5.6cvs, protocol version 26)
#  GNU find and xargs
#  Perl 4 or Perl 5 for the html2htm.pl script
#  mkisofs (tested with 1.15a39)
# You may want to adjust some of the variables below.

set -eux
# source directory
: ${DIR=/pub/}
# temporary directory for creating trans.tbl files
: ${DIR0=/tmp/cd}
# title of the CD-ROM file system
: ${TITLE="FUNET CBM&CP/M"}
# the CD-ROM image
: ${IMAGE=/var/tmp/cd.iso}
# e-mail address of the image distributor
: ${EMAIL="$USER@$HOSTNAME"}
# the loop-back CD-ROM mount point
: ${CDROM="/mnt/cdrom"}

# GNU find (must know the -print0 option)
: ${FIND=find}
# GNU xargs (must know the -0 option)
: ${XARGS=xargs}
# loop-back mount command
: ${MOUNT="mount $IMAGE $CDROM -roloop"}
# loop-back umount command
: ${UMOUNT="umount $CDROM"}

rsync --exclude incoming -a rsync://rsync.nic.funet.fi/ftp/pub/cpm/ "$DIR"/cpm/
rsync --exclude incoming --exclude incoming.old --exclude Archaeology \
 --delete-excluded -a rsync://rsync.nic.funet.fi/ftp/pub/cbm/ "$DIR"/cbm/

# create the Windows autorun files if needed
[ -f "$DIR"/autorun.inf ] || {
    cat >> "$DIR"/autorun.inf << EOF
[autorun]
OPEN=Wscript autorun.js //B //nologo
EOF
}
[ -f "$DIR"/autorun.js ] || {
    cat >> "$DIR"/autorun.js << EOF
WshShell = WScript.CreateObject("WScript.Shell");
WshShell.Run("index.html", 1, 0);
WScript.DisconnectObject(WshShell);
EOF
}
[ -f "$DIR"/index.html ] || {
    cat >> "$DIR"/index.html << EOF
<?xml version="1.0"?><!-- -*- xml -*- -->
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
 "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html
 xmlns="http://www.w3.org/1999/xhtml"
 lang="en" xml:lang="en" dir="ltr">

<head>
<title>ftp.funet.fi:/pub/cbm/&amp;/pub/cpm/</title>
<link rev="made" href="mailto:cbm-adm@ftp.funet.fi" />
<meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1" />
</head><body>
<h1>ftp://ftp.funet.fi/pub/cbm/&amp;/pub/cbm/</h1>
<dl>
 <dt><a href="cbm/index.html"><acronym
 title="Commodore Business Machines">CBM</acronym></a></dt>
 <dt><a href="cpm/index.html"><acronym
 title="Control Program for Microcomputers">CP/M</acronym></a></dt>
</dl>
</body></html>
EOF
}

[ "$DIR" = "$DIR0" ] || {
    [ -d "$DIR0" ] && rm -fr "$DIR0"
    mkdir "$DIR0"
    # create a directory tree with the names (to generate TRANS.TBL faster)
    (
	cd "$DIR"
	"$FIND" . -type d \! -name . \! -name .. -print0
    ) | (
	cd "$DIR0"
	"$XARGS" -0 mkdir
    )
    (
	cd "$DIR"
	"$FIND" . \! -type d -print0
    ) | (
	cd "$DIR0"
	"$XARGS" -0 touch
    )
}

# Create the initial image to get access to the TRANS.TBL files
mkisofs -o "$IMAGE" -p "$EMAIL" -V "$TITLE" -J -r -T "$DIR0"
# Remove the temporary directory tree
[ "$DIR" = "$DIR0" ] || rm -fr "$DIR0"
# Mount the image
$MOUNT
# Create HTML files for short filenames
: ${H2H="`pwd`/html2htm.pl"}
cd "$DIR"
perl "$H2H" "$CDROM" \
    index.html IX.HTM ALLFILES.html A_FILES.HTM ALLDIRS.html A_DIRS.HTM
$UMOUNT
# Create the final image
mkisofs -o "$IMAGE" -p "$EMAIL" -V "$TITLE" -J -r -T .
